@@ -21,6 +21,7 @@ import gnssrefl.rinpy as rinpy
 import gnssrefl.gpssnr as gpssnr
 import gnssrefl.gnsssnr as gnsssnr 
 
+import gnssrefl.gnsssnrbigger as gnsssnrbigger
 
 class constants:
     omegaEarth = 7.2921151467E-5 #      %rad/sec
@@ -59,7 +60,8 @@ def run_rinex2snr(station, year_list, doy_list, isnr, orbtype, rate,dec_rate,arc
     srate - integer sample rate, for RINEX 3 only.
     2021mar20 check for illegal day of years ....
     2021aug01 added mk option for uppercase file names per makan karegar request
-    2021aug15v added weekly option
+    2021aug15 added weekly option
+    2021nov08 making nolook work for rinex 3 files ...
     """
     # do not allow illegal skipit values
     if skipit < 1:
@@ -71,6 +73,10 @@ def run_rinex2snr(station, year_list, doy_list, isnr, orbtype, rate,dec_rate,arc
         version = 2
         if not mk:
             station = station.lower()
+    elif (NS == 6 and archive == 'jp') :
+        version = 2
+        if not mk:
+            station = station[-4:].upper()
     elif (NS == 9):
         #print('Assume RINEX 3'); 
         version = 3
@@ -125,8 +131,8 @@ def run_rinex2snr(station, year_list, doy_list, isnr, orbtype, rate,dec_rate,arc
             if (not illegal_day) and (not snre):
                 r = station + cdoy + '0.' + cyy + 'o'
                 rgz = station + cdoy + '0.' + cyy + 'o.gz'
-                print('Will seek RINEX file ', station, ' year:', year, ' doy:', doy, ' translate with ', translator, ' at the ', archive, ' archive')
                 if nol:
+                    print('Will assume RINEX file ', station, ' year:', year, ' doy:', doy, 'is in the local directory')
                     # this assumes RINEX file is in local directory or "nearby"
                     if version == 2:
                         the_makan_option(station,cyyyy,cyy,cdoy) # looks everywhere in your local directories
@@ -137,14 +143,15 @@ def run_rinex2snr(station, year_list, doy_list, isnr, orbtype, rate,dec_rate,arc
                             print('You Chose the No Look Option, but did not provide the needed RINEX file.')
                     if version == 3:
 ##                        print(' 2021feb22 using sample rate explicitly')
-                        # this is going to fail when srate < 10
-                        r3 = station9ch + '_R_' + str(year) + cdoy + '0000_01D_' + str(srate) + 'S_MO.rnx'
-                        r3gz = station9ch + '_R_' + str(year) + cdoy + '0000_01D_' + str(srate) + 'S_MO.rnx.gz'
+                        # make sure sample rate takes up two characters ...
+                        csrate = '{:02d}'.format(srate)
+                        r3 = station9ch + '_R_' + str(year) + cdoy + '0000_01D_' + csrate + 'S_MO.rnx'
+                        r3gz = station9ch + '_R_' + str(year) + cdoy + '0000_01D_' + csrate + 'S_MO.rnx.gz'
                         r2 = station + cdoy + '0.' + cyy + 'o'
                         if os.path.exists(r3gz):
                             subprocess.call(['gunzip', r3gz])
                         if os.path.exists(r3):
-                            #print('RINEX 3 file exists locally')
+                            print('The RINEX 3 file exists locally')
                             fexists = g.new_rinex3_rinex2(r3,r2)
                             if fexists:
                                 #print('Rinex 3 to 2 conversion worked, now convert to snr format')
@@ -153,8 +160,10 @@ def run_rinex2snr(station, year_list, doy_list, isnr, orbtype, rate,dec_rate,arc
                                 print('Something about the RINEX 3-2 conversion did not work')
                         else:
                             print('You Chose the No Look Option, but did not provide the needed RINEX3 file.')
+                            print('I assumed its name was ', r3)
 
                 else:
+                    print('Will seek the RINEX file externally for ', station, ' year:', year, ' doy:', doy, ' translate with ', translator, ' at the ', archive, ' archive')
                     if version == 3:
                         #  try unavco
                         rinex2exists = False; rinex3name = '';
@@ -274,7 +283,12 @@ def conv2snr(year, doy, station, option, orbtype,receiverrate,dec_rate,archive,f
                     if (orbtype  == 'gps') or (orbtype == 'nav'):
                         gpssnr.foo(in1,in2,in3,in4,in5,in6)
                     else:
-                        gnsssnr.foo(in1,in2,in3,in4,in5,in6)
+                        if (orbtype == 'ultra') or (orbtype == 'wum'):
+                            print('testing out things for orbit type ', orbtype)
+                            print('this file requires that more epochs be allowed.')
+                            gnsssnrbigger.foo(in1,in2,in3,in4,in5,in6)
+                        else:
+                            gnsssnr.foo(in1,in2,in3,in4,in5,in6)
                 else:
                     if (translator == 'fortran'):
                         t1=time.time()
@@ -664,9 +678,13 @@ def test_sp3(gpstime,sp3,systemsatlists,obsdata,obstypes,prntoidx,year,month,day
                             r=np.subtract(SatOrb,recv)
                             azimA = g.azimuth_angle(r, East, North)
                             eleA = g.elev_angle(up, r)*180/np.pi
+                            # 2021 october 26
+                            # thank you to andrea gatti for pointing out the mistake
                             if (eleA >= emin) and (eleA <= emax):
                                 fout.write("{0:3.0f} {1:10.4f} {2:10.4f} {3:10.0f} {4:7.2f} {5:7.2f} {6:7.2f} {7:7.2f} {8:7.2f} {9:7.2f} {10:7.2f} \n".format( 
-                                    prn+addon,eleA,azimA,Tp[ij]-gpssec0, 0,float(s6[ij]),s1[ij],float(s2[ij]),float(s5[ij]),float(s6[ij]),float(s7[ij]) ))
+                                    prn+addon,eleA,azimA,Tp[ij]-gpssec0, 0,float(s6[ij]),s1[ij],float(s2[ij]),float(s5[ij]),float(s7[ij]),float(s8[ij]) ))
+                                #fout.write("{0:3.0f} {1:10.4f} {2:10.4f} {3:10.0f} {4:7.2f} {5:7.2f} {6:7.2f} {7:7.2f} {8:7.2f} {9:7.2f} {10:7.2f} \n".format( 
+                                #    prn+addon,eleA,azimA,Tp[ij]-gpssec0, 0,float(s6[ij]),s1[ij],float(s2[ij]),float(s5[ij]),float(s6[ij]),float(s7[ij]) ))
                 else:
                     log.write('This satellite is not in the orbit file. {0:3.0f} \n'.format(prn))
         else:
@@ -808,8 +826,9 @@ def testing_sp3(gpstime,sp3,systemsatlists,obsdata,obstypes,prntoidx,year,month,
                                 azimA = g.azimuth_angle(r, East, North)
                                 eleA = g.elev_angle(up, r)*180/np.pi
                                 if (eleA >= emin) and (eleA <= emax):
+                                    # bug reported by Andrea Gatti. 2021 October 26
                                     fout.write("{0:3.0f} {1:10.4f} {2:10.4f} {3:10.0f} {4:7.2f} {5:7.2f} {6:7.2f} {7:7.2f} {8:7.2f} {9:7.2f} {10:7.2f} \n".format( 
-                                        prn+addon,eleA,azimA,Tp[ij]-gpssec0, 0,float(s6[ij]),s1[ij],float(s2[ij]),float(s5[ij]),float(s6[ij]),float(s7[ij]) ))
+                                        prn+addon,eleA,azimA,Tp[ij]-gpssec0, 0,float(s6[ij]),s1[ij],float(s2[ij]),float(s5[ij]),float(s7[ij]),float(s8[ij]) ))
                     else:
                         log.write('This satellite is not in the orbit file. {0:3.0f} \n'.format(prn))
         else:
