@@ -34,10 +34,13 @@ def print_badpoints(t,outliersize):
     f = 'outliers.txt'
     print('outliers written to file: ', f) 
     fout = open(f, 'w+')
-    for i in range(0,m):
-        fout.write('doy {0:3.0f} sat {1:3.0f} azim {2:6.2f} fr {3:3.0f} pk2noise {4:5.1f} residual {5:5.2f} \n'.format(
-            t[i,1], t[i,3],t[i,5], t[i,10], t[i,13], outliersize[i] ))
-    fout.close()
+    if (m > 0):
+        for i in range(0,m):
+            fout.write('doy {0:3.0f} sat {1:3.0f} azim {2:6.2f} fr {3:3.0f} pk2noise {4:5.1f} residual {5:5.2f} \n'.format( 
+                t[i,1], t[i,3],t[i,5], t[i,10], t[i,13], outliersize[i] ))
+        fout.close()
+    else:
+        print('no outlier points to write to a file')
 
 def output_names(txtdir, txtfile,csvfile,jsonfile):
     """
@@ -47,6 +50,9 @@ def output_names(txtdir, txtfile,csvfile,jsonfile):
     if writejson is true, then it is written to txtdir + jsonfile 
     if writecsv is true, then it is written to txtdir + csvfile and so on
     author: kristine larson
+
+    this is pretty dumb - should have one variable telling you the output you want (csv or json or txt)
+    the writejson option is not available
     """
     writetxt = True
     if txtfile == None:
@@ -73,7 +79,7 @@ def output_names(txtdir, txtfile,csvfile,jsonfile):
     print('outputfile ', outfile)
     return writetxt,writecsv,writejson,outfile
 
-def write_subdaily(outfile,station,ntv,writecsv,writetxt,extraline):
+def write_subdaily(outfile,station,ntv,writecsv,extraline,**kwargs):
     """
     input: output filename
     station - 4 character station name
@@ -85,16 +91,29 @@ def write_subdaily(outfile,station,ntv,writecsv,writetxt,extraline):
     mistake.  put m,d,h,m,s at the end
 
     author: kristine larson
+
+    this does not accommodate json as yet
     """
     # this is lazy - should use shape
+    RHdot_corr= kwargs.get('RHdot_corr',[])
+    newRH = kwargs.get('newRH',[])
+    if len(RHdot_corr) + len(newRH) == 0:
+        extra  = False
+    else:
+        print('extra columns are being written')
+        extra = True
     N= len(ntv)
     nr,nc = ntv.shape
-    print('write_subdaily', nr,nc)
+    if nr == 0:
+        print('No results in this file, so nothing to write out.')
+        return
+    print(nr, ' observations will be written to ',outfile)
     N= nr
-    # this is true so I don't have to move the indents
-    print('Results are being written to : ', outfile)
     fout = open(outfile, 'w+')
-    write_out_header(fout,station,extraline)
+    if extra:
+        write_out_header(fout,station,extraline,extra_columns=True)
+    else:
+        write_out_header(fout,station,extraline)
     dtime = False
     for i in np.arange(0,N,1):
         year = int(ntv[i,0]); doy = int(ntv[i,1])
@@ -106,61 +125,82 @@ def write_subdaily(outfile,station,ntv,writecsv,writetxt,extraline):
         #minute = ctime[3:5]
         # you can either write a csv or text, but not both
         if writecsv:
-            fout.write(" {0:4.0f},{1:3.0f},{2:6.3f},{3:3.0f},{4:6.3f},{5:6.2f},{6:6.2f},{7:6.2f},{8:6.2f},{9:4.0f},{10:3.0f},{11:2.0f},{12:8.5f},{13:6.2f},{14:7.2f},{15:12.6f},{16:1.0f},{17:2.0f},{18:2.0f},{19:2.0f},{20:2.0f},{21:2.0f} \n".format(year, doy, rh,ntv[i,3],UTCtime,ntv[i,5],ntv[i,6],ntv[i,7],ntv[i,8], ntv[i,9], \
+            fout.write(" {0:4.0f},{1:3.0f},{2:7.3f},{3:3.0f},{4:6.3f},{5:6.2f},{6:6.2f},{7:6.2f},{8:6.2f},{9:4.0f},{10:3.0f},{11:2.0f},{12:8.5f},{13:6.2f},{14:7.2f},{15:12.6f},{16:1.0f},{17:2.0f},{18:2.0f},{19:2.0f},{20:2.0f},{21:2.0f} \n".format(year, doy, rh,ntv[i,3],UTCtime,ntv[i,5],ntv[i,6],ntv[i,7],ntv[i,8], ntv[i,9], \
                             ntv[i,10],ntv[i,11], ntv[i,12],ntv[i,13], ntv[i,14], ntv[i,15], ntv[i,16],month,day,hour,minute, int(second)))
         else:
-            fout.write(" {0:4.0f} {1:3.0f} {2:6.3f} {3:3.0f} {4:6.3f} {5:6.2f} {6:6.2f} {7:6.2f} {8:6.2f} {9:4.0f} {10:3.0f} {11:2.0f} {12:8.5f} {13:6.2f} {14:7.2f} {15:12.6f} {16:1.0f} {17:2.0f} {18:2.0f} {19:2.0f} {20:2.0f} {21:2.0f} \n".format(year, doy, rh,ntv[i,3],UTCtime,ntv[i,5],ntv[i,6],ntv[i,7],ntv[i,8], ntv[i,9], \
+            if extra:
+                fout.write(" {0:4.0f} {1:3.0f} {2:7.3f} {3:3.0f} {4:6.3f} {5:6.2f} {6:6.2f} {7:6.2f} {8:6.2f} {9:4.0f} {10:3.0f} {11:2.0f} {12:8.5f} {13:6.2f} {14:7.2f} {15:12.6f} {16:1.0f} {17:2.0f} {18:2.0f} {19:2.0f} {20:2.0f} {21:2.0f} {22:6.3f} {23:6.3f} \n".format(year, doy, rh,ntv[i,3],UTCtime,ntv[i,5],ntv[i,6],ntv[i,7],ntv[i,8], ntv[i,9], \
+                            ntv[i,10],ntv[i,11], ntv[i,12],ntv[i,13], ntv[i,14], ntv[i,15], ntv[i,16],month,day,hour,minute, int(second), RHdot_corr[i], newRH[i]))
+            else:
+                fout.write(" {0:4.0f} {1:3.0f} {2:7.3f} {3:3.0f} {4:6.3f} {5:6.2f} {6:6.2f} {7:6.2f} {8:6.2f} {9:4.0f} {10:3.0f} {11:2.0f} {12:8.5f} {13:6.2f} {14:7.2f} {15:12.6f} {16:1.0f} {17:2.0f} {18:2.0f} {19:2.0f} {20:2.0f} {21:2.0f} \n".format(year, doy, rh,ntv[i,3],UTCtime,ntv[i,5],ntv[i,6],ntv[i,7],ntv[i,8], ntv[i,9], \
                             ntv[i,10],ntv[i,11], ntv[i,12],ntv[i,13], ntv[i,14], ntv[i,15], ntv[i,16],month,day,hour,minute, int(second)))
     fout.close()
 
 
-def readin_and_plot(station, year,d1,d2,plt2screen,extension):
+def readin_and_plot(station, year,d1,d2,plt2screen,extension,sigma,writecsv,azim1,azim2,ampl,peak2noise,txtfile):
     """
     Inputs:
     station - 4 character name
     year is year ;-)  
-    d1 and d2 are days of year if you want to look at a smaller dataset
+    d1 and d2 are days of year if you want to look at a smaller dataset (integers)
     plt2screen is a boolean whether you want the plot displayed to the screen
+    extension is where the results files stored in that subdirectory ('' for default) 
+    sigma is how many standard deviations away from mean you allow.   (float)
+
+    files now written out here rather than from subdaily_cl.py
 
     author: kristine larson
     2021april27 return datetime object
+    added azimuth, amplitude,peak2noise constraints
+    allow file to be input
     """
-    print('Read in the RH retrievals and plot it between these days: ',d1,d2)
-    if (d2 < d1):
-        print('First day of year must be less than last day of year. Exiting')
-        sys.exit()
-
+    # fontsize
+    fs = 12
     xdir = os.environ['REFL_CODE']
-
     # output will go to REFL_CODE/Files
     txtdir = xdir + '/Files'
+    print('Will remove daily outliers greater than ', sigma, ' sigma')
     if not os.path.exists(txtdir):
         os.makedirs(txtdir)
 
+    print('>>>>>>>>>>>>>>>>>>>>>>>> readin RH data <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<')
+    if txtfile == '':
+        print('Read in the RH retrievals for ', year, ' and these days: ',d1,d2)
+        if (d2 < d1):
+            print('First day of year must be less than last day of year. Exiting')
+            sys.exit()
     # where the LSP results are kept
-    direc = xdir + '/' + str(year) + '/results/' + station  + '/' + extension + '/'
+        direc = xdir + '/' + str(year) + '/results/' + station  + '/' + extension + '/'
     # datetime object for time
-    obstimes = []
-    tv = np.empty(shape=[0, 17])
-    if os.path.isdir(direc):
-        all_files = os.listdir(direc)
-        #print(direc)
-        #print('Number of files in ', year, len(all_files))
-        for f in all_files:
+        obstimes = []
+        tv = np.empty(shape=[0, 17])
+        if os.path.isdir(direc):
+            all_files = os.listdir(direc)
+            for f in all_files:
             # only evaluate txt files
-            if (len(f) ==  7) and (f[4:8] == 'txt'):
-                day = int(f[0:3])
-                if (day >= d1) & (day <= d2):
-                    fname = direc + f
-                    try:
-                        a = np.loadtxt(fname,comments='%')
-                        if len(a) > 0:
-                            tv = np.append(tv, a,axis=0)
-                    except:
-                        print('some issue with ',fname)
+                if (len(f) ==  7) and (f[4:7] == 'txt'):
+                    day = int(f[0:3])
+                #print('looking at file: ', day)
+                    if (day >= d1) & (day <= d2):
+                        fname = direc + f
+                        try:
+                            a = np.loadtxt(fname,comments='%')
+                            if len(a) > 0:
+                                tv = np.append(tv, a,axis=0)
+                        except:
+                            print('some issue with ',fname)
+
+    else:
+        # using external file of concatenated results
+        tv = np.loadtxt(txtfile,comments='%')
 
     nr,nc=tv.shape
-    print('Number of RH retrievals', nr)
+    tvoriginal = tv
+    print('Number of initial RH retrievals', nr)
+    nroriginal = nr
+    if (nr == 0):
+        print('Exiting')
+        sys.exit()
 
     t=tv[:,0] + (tv[:,1] + tv[:,4]/24)/365.25
     rh = tv[:,2]
@@ -168,16 +208,38 @@ def readin_and_plot(station, year,d1,d2,plt2screen,extension):
     # sort the data
     ii = np.argsort(t)
     t = t[ii] ; rh = rh[ii]
-    # store it all in a new variable
+    # store the sorted data 
     tv = tv[ii,:]
 
+    # apply azimuth constraints 
+    ii = (tv[:,5]  >= azim1) & (tv[:,5] <= azim2)
+    tv = tv[ii,:]
+    print(nr-len(tv) , ' points removed for azimuth constraints ',azim1,azim2)
+
+    # now apply amplitude constraint
+    nr,nc = tv.shape
+    ii = (tv[:,6]  >= ampl) ; tv = tv[ii,:]
+    print(nr-len(tv) , ' points removed for amplitude constraint ')
+
+    # now apply peak2noise constraint
+    nr,nc = tv.shape
+    ii = (tv[:,13]  >= peak2noise) ; tv = tv[ii,:]
+    print(nr-len(tv) , ' points removed for peak2noise constraints ')
+
 
     # silly - why read it if you are not going to use it
     # and restrict by doy - mostly to make testing go faster
     ii = (tv[:,1] >= d1) & (tv[:,1] <= d2)
     tv = tv[ii,:]
+    firstdoy = int(min(tv[:,1]))
+    lastdoy =  int(max(tv[:,1]))
+
+    tvoriginal = tv
+    nr,nc = tvoriginal.shape
+    print('RH retrievals after all commandline constraints', nr)
 
     # now that you have everything the way you want it .... 
+    # make the datatime objects
     nr,nc = tv.shape
     otimes = []
     for ijk in range(0,nr):
@@ -185,43 +247,130 @@ def readin_and_plot(station, year,d1,d2,plt2screen,extension):
         otimes.append(dtime)
 
     # make arrays to save number of RH retrievals on each day
-    nval = []; tval = []; y = tv[0,0]
-    for d in range(d1,(d2+1)):
+    residuals = np.empty(shape=[0,1])
+    nval = []; tval = []; y = tv[0,0]; 
+    stats = np.empty(shape=[0,3])
+    for d in range(firstdoy,(lastdoy+1)):
         ii = (tv[:,1] == d)
         dtime, iyear,imon,iday,ihour,imin,isec = g.ymd_hhmmss(year,d,12,True)
         tval.append(dtime)
         n = len(tv[ii,1])
         nval.append(n)
+        if (n > 0):
+            rhavg = np.mean(tv[ii,2]); 
+            rhstd = np.std(tv[ii,2]); 
+            newl = [dtime, rhavg, rhstd]
+            stats = np.append(stats, [newl], axis=0)
+            b = ( tv[ii,2] - rhavg*np.ones( len(tv[ii,2]) ))/ rhstd
+            residuals = np.append(residuals, b)
     #
-    fs = 12
+
+
+    ii = (np.absolute(residuals) > sigma)
+    jj = (np.absolute(residuals) < sigma)
     if plt2screen:
         fig,ax=plt.subplots()
-        #plt.figure()
-        ax.plot(tval,nval,'.')
+        ax.plot(tval,nval,'bo')
         plt.title(station + ': number of RH retrievals each day',fontsize=fs)
         plt.xticks(rotation =45,fontsize=fs)
         plt.yticks(fontsize=fs)
         plt.grid()
         fig.autofmt_xdate()
 
+        minAz = float(np.min(tv[:,5])) ; maxAz = float(np.max(tv[:,5]))
         fig,ax=plt.subplots()
-        ax.plot( otimes, tv[:,2], '.')
+        # put some azimuth information on it
+        colors = tv[:,5]
+        # ax.plot( otimes, tv[:,2], '.')
+        # https://matplotlib.org/stable/gallery/lines_bars_and_markers/scatter_with_legend.html
+        scatter = ax.scatter(otimes,tv[:,2],marker='o', s=15, c=colors)
+        legend1 = ax.legend(*scatter.legend_elements(), bbox_to_anchor = (1.01, 0.8),title="Azim")
+        ax.add_artist(legend1)
         plt.ylabel('meters',fontsize=fs)
-        plt.title('GNSS station: ' + station + ' Reflector Heights', fontsize=fs)
+        plt.title('GNSS station: ' + station.upper() + ' Reflector Heights (azimuth)', fontsize=fs)
         plt.gca().invert_yaxis()
-        plt.xticks(rotation =45,fontsize=fs)
-        plt.yticks(fontsize=fs)
-        plt.grid()
-        fig.autofmt_xdate()
+        plt.xticks(rotation =45,fontsize=fs); plt.yticks(fontsize=fs)
+        plt.grid() ; fig.autofmt_xdate()
 
-        plotname = txtdir + '/' + station + '_subdaily_RH.png'
+        plotname = txtdir + '/' + station + '_subdaily_azim_RH.png'
         plt.savefig(plotname,dpi=300)
         print('png file saved as: ', plotname)
-        plt.show()
-    # probably nice to also have a plot with number of retrievals vs time
 
+        fig,ax=plt.subplots()
+        # put some amplitude information on it
+        colors = tv[:,6]
+        # ax.plot( otimes, tv[:,2], '.')
+        # https://matplotlib.org/stable/gallery/lines_bars_and_markers/scatter_with_legend.html
+        scatter = ax.scatter(otimes,tv[:,2],marker='o', s=15, c=colors)
+        legend1 = ax.legend(*scatter.legend_elements(), bbox_to_anchor = (1.01, 0.8),title="Ampl")
+        ax.add_artist(legend1)
+        plt.ylabel('meters',fontsize=fs)
+        plt.title('GNSS station: ' + station.upper() + ' Reflector Heights (amplitude)', fontsize=fs)
+        plt.gca().invert_yaxis()
+        plt.xticks(rotation =45,fontsize=fs); plt.yticks(fontsize=fs)
+        plt.grid() ; fig.autofmt_xdate()
+
+        plotname = txtdir + '/' + station + '_subdaily_ampl_RH.png'
+        plt.savefig(plotname,dpi=300)
+        print('png file saved as: ', plotname)
+
+        # poor man's outlier detector
+        fig,ax=plt.subplots()
+        colors = tv[:,5]
+        # put some amplitude information on it
+        # ax.plot( otimes, tv[:,2], '.')
+        # https://matplotlib.org/stable/gallery/lines_bars_and_markers/scatter_with_legend.html
+        otimesarray = np.asarray(otimes)
+        plt.plot(otimes,tv[:,2], '.',color='gray',label='arcs')
+        plt.plot(stats[:,0], stats[:,1], 'o',markersize=8,color='blue',label='daily avg')
+        slabel = str(sigma) + ' sigma'
+        plt.plot(stats[:,0], stats[:,1]-sigma*stats[:,2], '--',color='black',label=slabel)
+        plt.plot(stats[:,0], stats[:,1]+sigma*stats[:,2], '--',color='black')
+        plt.plot(otimesarray[ii],tv[ii,2], '.',color='red',label='outliers',markersize=12)
+        plt.legend(loc="upper left")
+        plt.ylabel('meters',fontsize=fs)
+        plt.title('GNSS station: ' + station.upper() + ' Reflector Heights', fontsize=fs)
+        plt.gca().invert_yaxis()
+        plt.xticks(rotation =45,fontsize=fs); plt.yticks(fontsize=fs)
+        plt.grid() ; fig.autofmt_xdate()
+
+        plotname = txtdir + '/' + station + '_outlier_hunting.png'
+        plt.savefig(plotname,dpi=300)
+        print('png file saved as: ', plotname)
+
+        fig,ax=plt.subplots()
+        plt.plot(otimesarray[jj],tv[jj,2], '.',color='green',label='arcs')
+        plt.gca().invert_yaxis()
+        plt.ylabel('meters',fontsize=fs)
+        plt.xticks(rotation =45,fontsize=fs); plt.yticks(fontsize=fs)
+        plt.title('GNSS station: ' + station.upper() + ' Edited Reflector Heights', fontsize=fs)
+        plt.grid() ; fig.autofmt_xdate()
+        plotname = txtdir + '/' + station + '_outliers_removed.png'
+        plt.savefig(plotname,dpi=300)
+        print('png file saved as: ', plotname)
+
+        plt.show()
+
+    # this might work... and then again, it might not
+    print_badpoints(tv[ii,:],residuals[ii])
+
+    # now write things out
+    fname = xdir + '/Files/' + station + '_subdaily_rh.txt'
+    fname_new = xdir + '/Files/' + station + '_subdaily_rh_edits.txt'
+    extraline = ''
+
+    editedtv = tv[jj,:]
+    nr,nc = editedtv.shape
+
+    #print('Observations after constraints ', nroriginal)
+    write_subdaily(fname,station,tvoriginal,writecsv,extraline)
+    print('Edited observations',nr)
+    write_subdaily(fname_new,station,editedtv,writecsv,extraline)
+    NV = len(tvoriginal)
+    print('Percent of observations removed: ', round(100*(NV-nr)/NV,2))
     
-    return tv,otimes
+    # now return the names of the output files ... 
+    return tv,otimes, fname, fname_new
 
 
 def quickTr(year, doy,frachours):
@@ -294,21 +443,26 @@ def in_out(x,y):
 
     return x,spline(x) 
     
-def write_out_header(fout,station,extraline):
+def write_out_header(fout,station,extraline,**kwargs):
     """
     writes out header for results file ... 
     21may04 extra line for user
     changed this so that it is EXACTLY THE SAME as gnssir, with extra columns for m/d/h/m
     author: kristine larson
     """
+    extra_columns = kwargs.get('extra_columns',False)
     xxx = str(datetime.datetime.now().strftime("%Y-%m-%d %H:%M"))
     fout.write('% Results for {0:4s} calculated on {1:20s} \n'.format(  station, xxx ))
     fout.write('% gnssrefl, https://github.com/kristinemlarson \n')
     if len(extraline) > 0:
         fout.write('% IMPORTANT {0:s} \n'.format(  extraline ))
     fout.write('% Phase Center corrections have NOT been applied \n')
-    fout.write("% (1)  (2)   (3) (4)  (5)     (6)   (7)    (8)    (9)   (10)  (11) (12) (13)    (14)     (15)    (16) (17) (18,19,20,21,22)\n")
-    fout.write("% year, doy, RH, sat,UTCtime, Azim, Amp,  eminO, emaxO,NumbOf,freq,rise,EdotF, PkNoise  DelT     MJD  refr  MM DD HH MM SS \n")
+    if extra_columns:
+        fout.write("% (1)  (2)   (3) (4)  (5)     (6)   (7)    (8)    (9)   (10)  (11) (12) (13)    (14)     (15)    (16) (17) (18,19,20,21,22) (23)    (24)\n")
+        fout.write("% year, doy, RH, sat,UTCtime, Azim, Amp,  eminO, emaxO,NumbOf,freq,rise,EdotF, PkNoise  DelT     MJD  refr  MM DD HH MM SS  RHcor  newRH\n")
+    else:
+        fout.write("% (1)  (2)   (3) (4)  (5)     (6)   (7)    (8)    (9)   (10)  (11) (12) (13)    (14)     (15)    (16) (17) (18,19,20,21,22)\n")
+        fout.write("% year, doy, RH, sat,UTCtime, Azim, Amp,  eminO, emaxO,NumbOf,freq,rise,EdotF, PkNoise  DelT     MJD  refr  MM DD HH MM SS \n")
 
 
 def writejsonfile(ntv,station, outfile):
@@ -386,16 +540,17 @@ def writejsonfile(ntv,station, outfile):
 
 def splines_for_dummies2(station,fname,fname_new,perday,pltit,outlierV,**kwargs):
     """
-    tvd is a filename for subdaily RH results 
+    inputs:
+    station - 4 char
+    fname - input filename 
+    fname_new - output filename
+    perday is number of velocity measurements per day (interpolated)
+    pltit - boolean for plots to the screen
+    outlierV is meter outlier cutoff
 
     pltit is a boolean for plots to come to the screen
     note: x was in units of years before but now is in days??
 
-
-    Returns xx and y- these are in units of days (xx)
-    and meters (yy), where yy is the spline fit to reflector height
-    computed perday times per day
-
     2021april27 sending obstimes as kwarg input
     2021may5 change file format back to original file format
     author: kristine larson
@@ -403,28 +558,36 @@ def splines_for_dummies2(station,fname,fname_new,perday,pltit,outlierV,**kwargs)
     where did splines_for_dummies go? LOL
 
     21may18 try to remove massive outliers
+    21oct27 add usespline option because this code is not robust
 
     """
     fs = 12 # fontsize
+    print('>>>>>>>>>>>>>>>>>>>> Entering spline fit <<<<<<<<<<<<<<<<<<<<<<<<')
+    print('Input filename:', fname)
+    print('Output filename: ', fname_new)
     # read in the tvd values which are the output of gnssir
     # i.e. the reflector heights
     tvd = np.loadtxt(fname,comments='%')
+    if len(tvd) == 0:
+        print('empty input file')
+        return
     # sort the data by days 
     ii = np.argsort( (tvd[:,1]+tvd[:,4]/24) ).T
     tvd = tvd[ii,:]
 
-    print('try to remove massive outliers')
     NV = len(tvd)
+    # remove a median value
     medval = np.median(tvd[:,2])
     xx= tvd[:,2]-medval
     plt_histograms = False
+    # turned off these plots at request of jupyter notebook users 
     if plt_histograms:
         plt.figure()
         n, bins, patches = plt.hist(xx, 50, density=True, facecolor='g', alpha=0.75)
         plt.xlabel('standard deviations')
         plt.title('RH (median removed) ')
 
-    # use 3 sigma ... 
+    # use 3 sigma for the histogram plot
     Sig = np.std(xx)
     ij =  np.absolute(xx) < 3*Sig
     xnew = xx[ij]
@@ -438,6 +601,8 @@ def splines_for_dummies2(station,fname,fname_new,perday,pltit,outlierV,**kwargs)
         if pltit:
             plt.show()
 
+    # sure looks like 3 sigma is being removed here! But this would get rid of a real storm surge
+    # perhaps not needed since 3 sigma could be taken out from the daily values
     tvd = tvd[ij,:]
 
 
@@ -447,31 +612,35 @@ def splines_for_dummies2(station,fname,fname_new,perday,pltit,outlierV,**kwargs)
     h = tvd[:,2]
     # this is the edot factor
     xfac = tvd[:,12]
-
+    
     # now get obstimes if they were not passed
     obstimes = kwargs.get('obstimes',[])
     if len(obstimes) == 0:
-        print('you need to get those obstimes ...')
+        print('Calculating obstimes ...')
         obstimes = g.get_obstimes(tvd)
 
     # 
     fillgap = 1/24 # one hour fake values
     gap = 4/24 # up to four hour gap allowed
 
-    tnew =[] ; ynew =[]; 
+    tnew =[] ; ynew =[]; faket = []; 
     # fill in gaps using variables called tnew and ynew
+    Ngaps = 0
     for i in range(1,len(th)):
-        d= th[i]-th[i-1] # delta in time
+        d= th[i]-th[i-1] # delta in time in units of days ?
         if (d > gap):
             #print(t[i], t[i-1])
             x0 = th[i-1:i+1]
             h0 = h[i-1:i+1]
-            print('found a gap in hours',d*24, 'day ', x0[0])
-            f = scipy.interpolate.interp1d(x0,h0)
+            print('Gap on doy:', int(np.floor(x0[0])), ' lasting ', round(d*24,2), ' hours ')
+            Ngaps = Ngaps + 1
+            #f = scipy.interpolate.interp1d(x0,h0)
+            f = scipy.interpolate.interp1d(x0,h0,'quadratic')
             # so this is fake data
             ttnew = np.arange(th[i-1]+fillgap, th[i], fillgap)
             yynew = f(ttnew)
-            #print(ttnew)
+            print(ttnew)
+            faket = np.append(faket, ttnew)
             # now append it to your real data
             tnew = np.append(tnew,ttnew)
             ynew = np.append(ynew,yynew)
@@ -479,29 +648,55 @@ def splines_for_dummies2(station,fname,fname_new,perday,pltit,outlierV,**kwargs)
             tnew = np.append(tnew,th[i])
             ynew = np.append(ynew,h[i])
 
+    if (Ngaps > 3):
+        print('This is a beta version of the spline fit code - and does not work well with gaps. Exiting for your own safety.')
+        sys.exit()
     # sort it just to make sure ...
     ii = np.argsort( tnew) 
     tnew = tnew[ii]
     ynew = ynew[ii]
+#  should really just mirror it or remove data at the ends
+#   going to add more fake data ...
+    first = tnew[0]
+    last = tnew[-1]
+    ii = (tnew <= first + 0.1)
+    jj = (tnew >= last-0.1)
+    first6hours = np.mean(ynew[ii])
+    last6hours = np.mean(ynew[jj])
+    for ii in range(1,6):
+        tnew = np.append(tnew,first-ii/48)
+        ynew = np.append(ynew, first6hours)
+    for ii in range(1,6):
+        tnew = np.append(tnew,last+ii/48)
+        ynew = np.append(ynew, last6hours)
 
-    knots_per_day = 12
+    
+    # now sort them again .... 
+    ii = np.argsort(tnew) 
+    tnew = tnew[ii]
+    ynew = ynew[ii]
+
+    # making a knot every three hours ...
+    knots_per_day = 8
     Ndays = tnew.max()-tnew.min()
     numKnots = int(knots_per_day*(Ndays))
     print('First and last time values', '{0:8.3f} {1:8.3f} '.format (tnew.min(), tnew.max()) )
     print('Number of RH obs', len(h))
     print('Average obs per day', '{0:5.1f} '.format (len(h)/Ndays) )
     print('Number of knots: ', numKnots)
+    print('Outlier criterion with respect to spline fit (m): ', outlierV)
     print('Number of days of data: ', '{0:8.2f}'.format ( Ndays) )
     # need the first and last knot to be inside the time series
-    t1 = tnew.min()+0.05
-    t2 = tnew.max()-0.05
+    firstKnot_in_minutes = 15
+    t1 = tnew.min()+firstKnot_in_minutes/60/24
+    t2 = tnew.max()-firstKnot_in_minutes/60/24
     # try this 
     # 
     knots =np.linspace(t1,t2,num=numKnots)
 
     #ftest = open('testing.txt', 'w+')
-    #for i in range(0,len(tnew)):
-    #    ftest.write('{0:9.4f} {1:7.3f}  \n'.format( tnew[i], ynew[i]))
+    #for i in range(0,len(knots)):
+    #    ftest.write('{0:9.4f} \n'.format( knots[i]))
     #ftest.close()
 
     t, c, k = interpolate.splrep(tnew, ynew, s=0, k=3,t=knots,task=-1)
@@ -509,7 +704,8 @@ def splines_for_dummies2(station,fname,fname_new,perday,pltit,outlierV,**kwargs)
     # user specifies how many values per day you want to send back to the user  
 
     # should i do extrapolate True? it is the default  - could make it periodic?
-    spline = interpolate.BSpline(t, c, k, extrapolate=True)
+    #spline = interpolate.BSpline(t, c, k, extrapolate=True)
+    spline = interpolate.BSpline(t, c, k, extrapolate=False)
     # equal spacing in both x and y
     # evenly spaced data - units of days
     N = int(Ndays*perday)
@@ -533,20 +729,39 @@ def splines_for_dummies2(station,fname,fname_new,perday,pltit,outlierV,**kwargs)
         #obstimes = fract_to_obstimes(spl_x)
         plt.plot(spl_x, spl_y, 'r', label='spline')
         plt.title('Station: ' + station + ' Reflector Height')
-        plt.plot(th[j], h[j], 'co',label='Outliers') 
+        outlierstring = str(outlierV) + '(m) outliers'
+        plt.plot(th[j], h[j], 'co',label=outlierstring) 
         plt.ylabel('meters',fontsize=fs)
         plt.xlabel('days',fontsize=fs)
         plt.grid()
         plt.gca().invert_yaxis()
         plt.legend(loc="upper left")
 
-# take out the bad points
+
+# take out these points
     th = th[i]; h = h[i]
     xfac = xfac[i]
     resid_spl = resid_spl[i]
-# remove bad points from the original variable
     tvd = tvd[i,:]
 
+# should take out first and last six hours as well
+    t1 = float(th[0] + 6/24)
+    t2 = float(th[-1] - 6/24)
+# and again
+    i = (th >= t1)
+    th = th[i]; h = h[i]
+    xfac = xfac[i]
+    resid_spl = resid_spl[i]
+    tvd = tvd[i,:]
+
+    i = (th <= t2)
+    th = th[i]; h = h[i]
+    xfac = xfac[i]
+    resid_spl = resid_spl[i]
+    tvd = tvd[i,:]
+
+
+
 # put the original series without outliers eventually?
 # use first diff to get simple velocity.  
 # 24 hours a day, you asked for perday values by day
@@ -557,7 +772,11 @@ def splines_for_dummies2(station,fname,fname_new,perday,pltit,outlierV,**kwargs)
     yvel = obsPerHour*np.diff(spl_y)
 
     rhdot_at_th = np.interp(th, tvel, yvel)
-
+    #debug = open('debugging.txt', 'w+')
+    #for w in range(0,len(spl_x)):
+    #    debug.write(' {0:9.4f} {1:8.3f} \n'.format (spl_x[w], spl_y[w] ) )
+#
+#    debug.close()
     # this is the RHdot correction. This can be done better - 
     # this is just a start
     correction = xfac*rhdot_at_th
@@ -566,6 +785,12 @@ def splines_for_dummies2(station,fname,fname_new,perday,pltit,outlierV,**kwargs)
 
 
     if pltit:
+        fig,ax=plt.subplots()
+        ax.plot(th, correction,'.',label='RHcorr')
+        plt.title('RHdot Correction',fontsize=fs)
+        plt.ylabel('meters',fontsize=fs); 
+        plt.grid()
+
         fig,ax=plt.subplots()
         ax.plot(tvel, yvel, '-',label='RHdot')
         ax.plot(th, rhdot_at_th,'.',label='RHdot at obs')
@@ -575,12 +800,20 @@ def splines_for_dummies2(station,fname,fname_new,perday,pltit,outlierV,**kwargs)
         plt.grid()
         plt.yticks(fontsize=fs)
         plt.xticks(fontsize=fs)
+        ydiff = max(yvel) - min(yvel)
+        y2 = 0.1*ydiff + max(yvel)  ; y1 = min(yvel) - 0.1*ydiff
+        plt.xlim((th[0], th[-1]))
+        #print('min and max values', y1,y2,max(yvel), min(yvel))
+
+        #plt.ylim((y1,y2))
         #fig.autofmt_xdate()
 
         #plt.figure()
         fig,ax=plt.subplots()
-        ax.plot(th, resid_spl,'.',label='without RHdot corr')
-        ax.plot(th, resid_spl - correction,'.',label='with RHdot corr')
+        label1 = 'w/o RHdot ' + str( round(np.std(resid_spl),2)) + 'm'
+        label2 = 'w/ RHdot ' + str(round(np.std(resid_spl-correction),2)) + 'm'
+        ax.plot(th, resid_spl,'.',label= label1)
+        ax.plot(th, resid_spl - correction,'.',label=label2)
         plt.legend(loc="upper left")
         plt.xlabel('days of the year',fontsize=fs)
         plt.ylabel('meters',fontsize=fs)
@@ -601,12 +834,15 @@ def splines_for_dummies2(station,fname,fname_new,perday,pltit,outlierV,**kwargs)
         if len(ret) > 0:
             print('{0:3.0f} {1:6.2f} {2:6.2f} {3:6.0f}'.format (f, np.mean(ret), np.std(ret), len(ret) ) )
 
-    # put the correction in column 2
-    tvd[:,2] = tvd[:,2] - correction
-    writecsv = False
-    writetxt = True
-    extraline = 'Large outliers removed and RHDot correction has been applied'
-    write_subdaily(fname_new,station,tvd,writecsv,writetxt,extraline)
+    # put the correct RH in column 2
+    # but it would be good to keep the correction and the original value!
+    # and also chop off beginning and end
+    writecsv = False; writetxt = True
+    extraline = 'Large outliers removed/two exta columns '
+    newRH = tvd[:,2] - correction
+    write_subdaily(fname_new,station,tvd,writecsv,extraline,newRH=newRH, RHdot_corr=correction)
+    nr,nc = tvd.shape
+    print('Percent of observations removed:', round(100*(NV-nr)/NV,2))
 
     return tvd, correction 
 
@@ -636,3 +872,15 @@ def testing_hourly(tvd):
     plt.plot(t_hourly, rh_hourly,'.')
     plt.show()
 
+
+#        plt.figure()
+#        #plt.plot(obstimes, h, 'bo', label='Original points',markersize=3)
+#        plt.plot(th, h, 'bo', label='Original points',markersize=3)
+#        plt.title('Station: ' + station + ' Reflector Height')
+#        plt.ylabel('meters',fontsize=fs)
+#        plt.xlabel('days',fontsize=fs)
+#        plt.grid()
+#        plt.gca().invert_yaxis()
+#        plt.legend(loc="upper left")
+#        plt.show()
+
